/*
 * command.c
 *
 * The core of the client program.
 * This module executes the commands issued by the BSX-server.
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include "config.h"
#ifdef __TURBOC__
#include <dos.h>
#endif

struct desc { 	/* Structure describing a scene of several polys */
  char id[MAXIDLEN];
  struct polygon poly[MAXDESCSIZE];
  unsigned char descsize;
};

struct desc *dbase[DBASESIZE];		/* Last recently used scenes */

/* Interface to internet.c code */
extern unsigned char eat_hex();
extern unsigned char eat_char();
extern void tell_server();

/* Interface to scenery.c code */
extern void introduce_new_scene();
extern void add_object_to_scene();
extern void remove_object_from_scene();
extern void redraw_entire_scene();
extern void list_scene_contents();

/* interface to text.c code */
extern void download_counter();

/* interface to xtool.c code */
extern void place_sprite();

extern int done;

char logging=1;

/*-------------------------------------------------------------------------*/
/*      A U X I L A R Y     S T U F F                                      */
/*-------------------------------------------------------------------------*/

unsigned char get_slot() {      /* Find slot for a scene */
  static int slot = 0;
  if (++slot==DBASESIZE)
    slot = 0;
  return(slot);
}

/*-------------------------------------------------------------------------*/

int find_id(id)         /* Check wether desc is already known */
char *id;
{
  int s;

  for (s=0; s<DBASESIZE; s++)
    if (strcmp(id,dbase[s]->id)==0)
      return s;
  return(-1);
}

void depict_scene(scene,centrex,centrey,scale)
struct desc *scene;
unsigned char centrex, centrey;
int scale;
{
  unsigned char p, size, v;
  struct polygon translated_poly;

  for (p=0; p<scene->descsize; p++) {
    size = scene->poly[p].size;
    translated_poly.size = size;
    translated_poly.color= scene->poly[p].color;
    for (v=0; v<size; v++) {
      int tmpx, tmpy;

      tmpx = (((scene->poly[p].x[v]-127) /* * scale/100 */ )) + centrex;
      tmpy = (((scene->poly[p].y[v]-127) /* * scale/100 */ )) + centrey;
      if (tmpx>255) tmpx = 255;
      if (tmpy>255) tmpy = 255;
      if (tmpx<  0) tmpx =   0;
      if (tmpy<  0) tmpy =   0;
      translated_poly.x[v] = tmpx;
      translated_poly.y[v] = tmpy;
    }
    visualise_poly(translated_poly); 
  }
}

void lookup_and_depict_scene(entry,centrex,centrey,scale)
unsigned int entry;
unsigned char centrex;
unsigned char centrey;
int scale;
{
  if (entry<DBASESIZE)
    depict_scene(dbase[entry],centrex,centrey,scale);
}

/*-------------------------------------------------------------------------*/
/*      S E R V E R    P R O C E D U R E S                                 */
/*-------------------------------------------------------------------------*/

void serve_dfs() {
  unsigned char c, p, i, v;
  unsigned char id[MAXIDLEN];
  int s;
  int polysize,scenesize;

  /* define scene */
  for (i=0; ((c=eat_char()) != '.')&&(i<MAXIDLEN-1) ; i++)
    id[i] = c;
  id[i] = 0;
  s = find_id(id);
  if (s==-1)
    s = get_slot();
  for (i=0; i<MAXIDLEN; i++)
    dbase[s]->id[i] = id[i];
  scenesize = dbase[s]->descsize = eat_hex();
  if (scenesize > MAXDESCSIZE) {
    dbase[s]->descsize=0;
    return;	/* Noise on line -> ABORTION! */
  }
  for (p=0; p<scenesize; p++) {
    download_counter(scenesize-p-1);
    dbase[s]->poly[p].size = polysize = eat_hex();
    if (polysize>32) {
      dbase[s]->descsize=0;
      return;	/* Noise on line -> ABORTION! */
    }
    dbase[s]->poly[p].color = eat_hex();
    for (v = 0; v < polysize; v++) {
      dbase[s]->poly[p].x[v] = eat_hex();
      dbase[s]->poly[p].y[v] = eat_hex();
    }
  }
}

void serve_sce() {
  unsigned char i,c;
  unsigned char id[MAXIDLEN];
  int s;

  /* SCEne */
  for (i=0; ((c=eat_char()) != '.')&&(i<MAXIDLEN-1) ; i++)
    id[i] = c;
  id[i] = 0;
  s = find_id(id);
  if (s==-1) {
    tell_server("#RQS ");
    tell_server(id);
    tell_server("\n");
  } 
  introduce_new_scene(id);          /* Let the scene-admin know about this */
}

void serve_rfs() {
  redraw_entire_scene();
}

void serve_vio() {
  unsigned char c,i,position,depth;
  unsigned char id[MAXIDLEN];
  int s;

  for (i=0; ((c=eat_char()) != '.')&&(i<MAXIDLEN-1) ; i++)
    id[i] = c;
  id[i] = 0;
  s = find_id(id);
  if (s==-1) {
    tell_server("#RQO ");
    tell_server(id);
    tell_server("\n");
  } 
  position = eat_hex();
  if (position>15)
    position = 0;
  depth    = eat_hex();
  if (depth>7)
    depth = 7;
  add_object_to_scene(id,position,depth);
}

void serve_rmo() {
  unsigned char i,c,id[MAXIDLEN];

  for (i=0; ((c=eat_char()) != '.')&&(i<MAXIDLEN-1) ; i++)
    id[i] = c;
  id[i] = 0;
  remove_object_from_scene(id);
}

void serve_txt() {
  char mesg[800];
  unsigned char c;
  int i;

  for (i=0; ((c=eat_char()) != '$')&&(i<800-1) ; i++)
    mesg[i] = c;
  mesg[i] = 0;
  do_text(mesg);
}

void serve_tms() {
  done = 1;
}

void serve_pro() {
  putchar('\n');
  puts("o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o");
  puts("|							        |");
  puts("o  Thank you for your visit.				        o");
  puts("|  Regenesis would like to host for more visitors.	        |");
  puts("o  If you enjoyed this session, please spread the word.         o");
  puts("|  Invite your fellow users on your local system to Regenesis.  |");
  puts("o  Make your binary accessible for others.		        o");
  puts("|  and maybe even try a 'mail all' if ya can get away with it.  |");
  puts("o  Regenesis would be very grateful indeed.			o");
  puts("|                   				Bram Stolk	|");
  puts("o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o");
}

void serve_bom() {
  char dest[80],c;
  char letter[5000];
  char shell_cmd[100];
  char command[4], stop;
  char rep[80];
  int i;
  int size;
  FILE *f;

  i=0;
  do {
    c = eat_char();
    dest[i++] = c;
  } while (c!='$' && i<80);
  dest[i-1]=0;
  printf("Destination address = %s\n",dest);
  stop=0;
  size=0;
  do {
    c = eat_char();
    if (c=='@') {
      command[0] = eat_char();
      command[1] = eat_char();
      command[2] = eat_char();
      command[3] = 0;
      if (!strcmp(command,"EOM"))
       stop = 1;
      else {
       letter[size++] = command[0];
       letter[size++] = command[1];
       letter[size++] = command[2];
      }
    } else {
      letter[size++]=c;
    }
  } while (!stop && size<5000);
#ifdef UNIX_MAIL_AVAILABLE
  f = fopen("/tmp/bsxletter","wb");
  if (!f)
    return;
  for (i=0; i<size; i++)
    fputc(letter[i],f);
  fclose(f);
  sprintf(shell_cmd,"cat /tmp/bsxletter | mail %s ; rm /tmp/bsxletter",dest);
  printf("The bsx server just asked me to send mail to %s\n",dest);
  puts("Are you sure, you want this? Cause it might be a prank of Orion.");
  printf("Want to mail it? (Y/N) : "); fflush(stdout);
  fgets(rep,80,stdin);
  if (*rep=='Y' || *rep=='y') {
    printf("Executing unix command : %s\n",shell_cmd);
    system(shell_cmd);
  } else {
    puts("OK. fax command ignored!");
  }
#endif
}

void serve_lof() {
  logging = 0;
}

void serve_lon() {
  logging = 1;
}

void serve_pur() {
  int slot;
  unsigned char i,c,id[MAXIDLEN];

  for (i=0; ((c=eat_char()) != '.')&&(i<MAXIDLEN-1) ; i++)
    id[i] = c;
  id[i] = 0;
  slot = find_id(id);
  if (slot != -1)
    strcpy(dbase[slot]->id, "** PURGED **");
}

void serve_rqv() {
  char reply[80];

  sprintf(reply,"#VER %s%s\n",CLIENT_ID,VERSION);
  tell_server(reply);
}

/*
 * Module entry point
 *
 */

int process_command(command)
char *command;
{
  if ((strcmp(command,"DFS")==0) || (strcmp(command,"DFO")==0)) {
    /* define scene , or: define object */
    serve_dfs();
    return 1;
  }
  if (strcmp(command,"SCE")==0) {
    /* SCEne */
    serve_sce();
    return 1;
  }
  if (strcmp(command,"VIO")==0) {
    /* VIsualise Object */
    serve_vio();
    return 1;
  }
  if (strcmp(command,"RMO")==0) {
    /* ReMove Object */
    serve_rmo();
    return 1;
  }
  if (strcmp(command,"RFS")==0) {
    /* ReFreSh display */
    serve_rfs();
    return 1;
  }
  if (strcmp(command,"TXT")==0) {
    /* TeXT */
    serve_txt();
    return 1;
  }
  if (strcmp(command,"TMS")==0) {
    /* TerMinate Session */
    serve_tms();
    return 1;
  }
  if (strcmp(command,"LOF")==0) {
    /* Logging OFf */
    serve_lof();
    return 1;
  }
  if (strcmp(command,"LON")==0) {
    /* Logging ON */
    serve_lon();
    return 1;
  }
  if (strcmp(command,"PUR")==0) {
    /* PURge a dbase entry */
    serve_pur();
    return 1;
  }
  if (strcmp(command,"RQV")==0) {
    /* ReQuest Version of client */
    serve_rqv();
    return 1;
  }
  if (strcmp(command,"PRO")==0) {
    /* PROmote the system */
    serve_pro();
    return 1;
  }
  if (strcmp(command,"BOM")==0) {
    /* Begin of mail */
    serve_bom();
    return 1;
  }
  return 0;
}

void init_command_interpreter() {
  int i;

  for (i=0; i<DBASESIZE; i++) {
#ifdef __TURBOC__
    unsigned int segment;
    /*
     * Turbo C 2.0 has a very buggy malloc and farmalloc,
     * They allways screw up.... so let us switch to dos allocation.
     * If you know why the regular malloc in the #else-part won't
     * work, then please drop me a line at: stolk@fwi.uva.nl
     */
    if (allocmem((sizeof(struct desc)+15)/16,&segment)!=-1)
      error("Out of memory... try decreasing the value of DBASESIZE\n");
    dbase[i]=(struct desc *)MK_FP(segment,0);
#else
    dbase[i]=(struct desc *)malloc(sizeof(struct desc));
#endif
    if (!dbase[i]) error("Out of memory... try decreasing the value of DBASESIZE\n");
    memset(dbase[i],0,sizeof(struct desc));
  }
}

