#include <stdio.h>
#include <X11/Xlib.h>

#define NEWFONT

#ifdef NEWFONT

Font get_font(dpy,w,h)
     Display *dpy;
     int w,h;
{
  char specifier[10];

  sprintf(specifier, "%dx%d", w, h);
  return XLoadFont(dpy, specifier);
}

#else

char fontname[160];
int best_width;
int best_height;
char nr_of_fonts_that_comply=0;

/*
 * only consider fonts that have a constant width
 * that meets the required width.
 */
void check_font(name,f,width,height)
char *name;	/* name of the font */
XFontStruct *f;	/* specs on the font */
int width;	/* required width */
int height;	/* required width */
{
  int w,h;

  if (!f) return;

  if (f->min_bounds.width != f->max_bounds.width)
    return;	/* don't consider proportionaly spaced fonts */
  w = f->max_bounds.rbearing - f->min_bounds.lbearing;
  h = f->max_bounds.ascent   + f->max_bounds.descent;

  if (w<=width && h<=height) {
#if 0
    fprintf(stderr,"%s :\n",name);
    fprintf(stderr,"width = %d, height=%d\n",w,h);
#endif
    nr_of_fonts_that_comply++;
    if (best_width<width || ((best_width==width)&&(best_height<height))) {
      strcpy(fontname,name);
      best_height = h;
      best_width  = w;
    }
  }
}

/*
 * When I hardcode a font, I always get complaints through
 * email. People keep bugging me because of the unavailability
 * of the font I picked at their system.
 * That is why I want an intelligent font loader.
 * All I do is specifiy width and height, and the X11 server
 * should come up with one. Which one, I do not care, as long
 * as it's non-proportional, non-bold, and approximates but does
 * exceed the required width/height.
 */
Font get_font(dpy,w,h)
Display *dpy;
int w,h;
{
  char specifier[160];
  XFontStruct *infolist,*p;
  char **list,**q;
  int count;

  nr_of_fonts_that_comply=0;
  *fontname = 0;
  best_height=0;
  best_width=0;
  sprintf(specifier,"-*-*-medium-*-*-*-*-*-*-*-c-*-*-*");
  list = XListFontsWithInfo(dpy,specifier,128,&count,&infolist);
  if (!list || !count) {
    fprintf(stderr,
      "Not a single font from your X11 server meets the specs:\n%s\n",
      specifier
    );
    error("Could not load font.\n");
  }
  p = infolist;
  q = list;
  while (count--)
    check_font(*q++,p++,w,h);
  XFreeFontInfo(list,infolist,count);
  XFreeFontNames(list);
  if (!*fontname || !nr_of_fonts_that_comply)
    error("Could not load an appropriate font.");
#if 0
  fprintf(stderr,"Nr. of fonts that comply: %d\n",nr_of_fonts_that_comply); 
  fprintf(stderr,"Chosen font: %s\n",fontname);
#endif
  return XLoadFont(dpy,fontname);
}

#endif
