/*
 * Internet.c
 *
 * This module is primarily intended as an interface to internet.
 * eat_char and eat_hex get data from the BSX server via internet.
 * spit_char gets data from the console, and sends it to the
 * BSX-server via internet, as soon as a CR is received from the console.
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <stdio.h>
#include <termios.h>
#ifdef AIXV3
#include <sys/select.h>
#endif

extern int errno;

#define BUF_SIZE 1024

/*
 * Interface with text.c
 */
extern void add_char_to_prompt_line();
extern void add_char_to_text_window();
extern void add_string_to_text_window();

/*
 * Interface with xtool.c
 */
extern char suck_keyboard();

/*
 * Interface with socket.c
 */
extern int mud_socket;
extern char logging;

/*
 * Get a line from remmote BSX-server
 */

int get_line(buf, s) 
     char *buf;
     int   s;
{
  fd_set read_set;
  static struct timeval tout = { 0, 20000 };
  int read_result;

  while(1) {
    event_check();  /* Handle X-events */

    FD_ZERO(&read_set);
    FD_SET(mud_socket,&read_set); 
    
    if((read_result = select(mud_socket+1, &read_set, NULL, NULL, &tout)) < 0 
       && errno != EINTR)
	fatal("get_line: select: errno == %d\n", errno);

    if((read_result = read(mud_socket, buf, s)) < 0)
      if(errno == EINTR || errno == EWOULDBLOCK)
	continue;
      else
	fatal("get_line: read: errno == %d\n", errno);
    else 
      return read_result;
  }
}
      
/*
 * Get a char from the input buffer. If the buffer is empty fill it
 * with a call to get_line().
 */

unsigned char eat_char() 
{
  static unsigned char buf[BUF_SIZE];
  static int pos = 0;
  static int end = 0;
  
  while(1) {
    if(pos >= end) {
      end = get_line(buf, BUF_SIZE);
      if(end == 0) {         /* The remote server has closed the connection */
	extern void serv_tms();
	
	serve_tms();
	return ' ';
      }
      pos = 0;
    }
    if(buf[pos] < 128)
      return buf[pos++];
    pos++;
  }
}

/* Read server data and convert hex to dec */

unsigned char eat_hex() { 
  unsigned char lo,hi;

  hi = eat_char();
  lo = eat_char();
  if (hi > '9')
    hi = hi - 55;
  else
    hi = hi - 48;
  if (lo > '9')
    lo = lo - 55;
  else
    lo = lo - 48;
  return (hi*16+lo);
}

void tell_server(mesg)
     char *mesg;
{
  write(mud_socket, mesg, strlen(mesg));
}

