/*
 *
 *      Purpouse        :       Visualisation of BSX-MUD data
 *      Method          :       Polygon drawing
 *      Author          :       Bram Stolk
 *      Date            :       June 1991
 *	Last mod	:	Feb  1992
 *
 *      This program is a bsxmud client. It reads data sent by a
 *      bsxmud server. The data is then interpreted and sent to the
 *      screen. This should cause mudding never to be the same.
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include "config.h"
#include "options.h"

/* #define DEBUG */

#define DEFAULT_MUDNAME "bsxmud.lysator.liu.se"
#define DEFAULT_MUDNR "7475"

/* Interface to text.c */
extern void add_char_to_text_window();
extern void add_char_to_prompt_line();
extern void add_string_to_text_window();
extern char get_raw_char_from_keyboard();

/* Interface to internet.c code */
extern void spit_char();
extern unsigned char eat_char();

/* interface to command.c code */
extern int process_command();
extern void init_command_interpreter();

/* interface to xtool code */
extern int force;
extern int dont_do_motd;
extern int winscale;
extern int textarea;
extern void check_for_refresh();

int done;                               /* Marks end of session */
char *mudname;                          /* Internet name or number */
char *mudnr;                            /* Port number */

void usage()               /* needed by option processing macros */
{
  fprintf(stderr,"Usage: %s [-slmcd] [-t height] [[-a] host port]\n",program_name);
  exit(1);
}

/* ----------------- Main Program --------------------------------------*/

void main(argc,argv)
int  argc;
char *argv[];
{
  unsigned char c, command[4];

  /*
   * Assume some default options
   */
  force = 0;
  dont_do_motd = 0;
  winscale= 1;
  textarea= 25;	/* Fits really well, when using -l and a 1280x800 monitor */
  mudname = DEFAULT_MUDNAME;
  mudnr   = DEFAULT_MUDNR;

  startopts();
  noarg('s', winscale=1; textarea=25);
  noarg('l', winscale=2; textarea=12);
  noarg('m', force=1);
  noarg('c', force=2);
  noarg('d', dont_do_motd = 1);
  nargs('a', 2, mudname=ARG(1);mudnr=ARG(2));
  nargs('t', 1, textarea=atoi(ARG(1)));
  endopts();

  switch (argc)            /* process remaining arguments */
  {
    case 1 :
      break;
    case 3 :
      mudname = argv[1];
      mudnr = argv[2];
      break;
    default : usage();
  }

  done = 0;
  init_display();
  init_command_interpreter();
  connect_to_mud(mudname,mudnr);
  do {
    c = eat_char();
    if (c=='@') {
      command[0]=eat_char();
      command[1]=eat_char();
      command[2]=eat_char();
      command[3]=0;
      if (!process_command(command)) {
	add_char_to_text_window('@');
        add_string_to_text_window(command);
      }
    } else {
      add_char_to_text_window(c);
    }
  } while(!done);
  close_display();
  disconnect();
}
