/*
 * text.c
 *
 * This module administers the command line and the text screen.
 *
 */

#include "config.h"
#include <stdio.h>
#include <X11/Xlib.h>

#define UPDATE XFlush(dpy)

unsigned char xpos;		/* position of cursor in text area */
unsigned char prompt_xpos;      /* position of cursor on command line */
unsigned char init_done=0;
unsigned char text_colour;
unsigned char command_colour;
unsigned char noecho=0;

extern char need_refresh;
extern int textarea,winscale;
extern int BWMode;
extern Font font;
extern Display *dpy;
extern Window win;
extern GC contexts[16];

#define INPUTLINE 	(textarea-1)
#define COMMANDLINE 	(textarea+1)

/*
 * Wipes the 8 lines on and below the specified y-coordinate
 */
void wipe_line(y)
int y;
{ 
  XFillRectangle(dpy,win,contexts[15],0,winscale*(256+y*12)-1,512*winscale,12*winscale);
}

/*
 * Print_at simulation
 */
void print_at(x,y,c,col)
int x,y; /* position */
char c;  /* character */
char col;/* colour */
{
  XTextItem textitem;

  if (x>=85)
    return;
  textitem.chars = &c;
  textitem.nchars = 1;
  textitem.delta = 1;
  textitem.font = font;
  XDrawText(dpy,
            win,                /* Directly into window */
            contexts[col],
            6*winscale*x-1,
            256*winscale+y*12*winscale+8*winscale,
            &textitem,
            1);
}

/*
 * Front end for print_at to print entire strings instead of single chars
 */
void print_string_at(x,y,s,col)
int x,y;
char *s;
char col;
{
  while (*s) {
    print_at(x++,y,*s,col);
    s++;
  }
}

/*
 * Scrolls up the textwindow by 8 pixels.
 */
void scroll_up() {
  XCopyArea(dpy,
	    win,
	    win,
	    contexts[15],
	    0,
	    winscale*(256+12),
	    winscale*512,
	    winscale*((textarea-1)*12),
	    0,
	    winscale*256
  );
  wipe_line(INPUTLINE);
  xpos = 0;
  UPDATE;
}

/*
 * This is where the non-graphic data from the mudserver goes.
 */ 
void add_char_to_text_window(c)
char c;
{
  /* putchar(c); fflush(stdout); return; */
  c &= 127;			/* We want ASCII only */
  if (c==13 || c==10) { 	/* Handle CRs and Newlines */
    if (xpos>0) scroll_up();
    return;
  }
  if (c==8) {			/* Handle backspaces */
    if (xpos) {
      xpos--;
      print_at(xpos,INPUTLINE,' ',text_colour);
    }
    return;
  }
  if (c==9) {			/* Handle TAB chars .... assume 8 spacing */
    unsigned char t;
    t = ((xpos>>3)+1)<<3;
    while ((xpos < t) && (xpos<80))
      print_at(xpos++,INPUTLINE,' ',text_colour);
    return;
  }
  if (c==1) {
    noecho++;
    return;
  }
  if (c<32)
    c='*';
  if (xpos==80)
    scroll_up();
  print_at(xpos,INPUTLINE,c,text_colour);
  xpos++;
  if (c==':')
    UPDATE;
}

/*
 * string version of add_char_to_text_window
 * ..for the programmer's convenience.
 */
void add_string_to_text_window(s)
char *s;
{
  while (*s) {
    add_char_to_text_window(*s);
    s++;
  }
  UPDATE;
}

/*
 * Let the user see what (s)he just typed
 */
void add_char_to_prompt_line(c)
char c;
{
  if (noecho==1)
    return;
  if (c==8) {
    if (prompt_xpos) {
      prompt_xpos--;
      XFillRectangle(dpy,win,contexts[15],
		     prompt_xpos*6*winscale,winscale*(256+COMMANDLINE*12)-1,
		     winscale*6+1,12*winscale);
      UPDATE;
    }
    return;
  }
  if ((c==13 || c==10) && (prompt_xpos!=0)) {
    prompt_xpos = 0;
    wipe_line(COMMANDLINE);
    UPDATE;
    return;
  }
  if (c<32)
    return;
  if (prompt_xpos==80)
    return;
  print_at(prompt_xpos++,COMMANDLINE,c,command_colour);
  UPDATE;
}

char s[20] = " downloading: -- ";

/*
 * Redraw screen
 */
void redraw_screen() {
  int i,m;
  char *t;

  m = '=';
  for (i=0; i<85; i++)
    if (i<60 || i>77)
      print_at(i,textarea,m,0);
  t=s;
  i=60;
  while (*t) {
    print_at(i,textarea,*t,0);
    i++;
    t++;
  }
  if (!init_done) {
    xpos = 0;
    prompt_xpos = 0;
    if (BWMode) {
      text_colour = 0;
      command_colour = 0;
    } else {
      text_colour = 4;
      command_colour = 1;
    }
    add_string_to_text_window("BSX client for X-11\nversion ");
    add_string_to_text_window(VERSION);
    add_string_to_text_window("\n(c)1991-93 by Bram Stolk.\nUse the -h option for a help screen.\n");
    init_done = 1;
  }
}

/*
 * Some code to implement the wait counters, to calm down the users
 */

void download_counter(c)
unsigned char c;
{
  char hi,lo;

  if (c==0) {
    lo = hi = '-';
  } else {
    hi = c/10;
    if (hi)
      hi = hi + '0';
    else
      hi = ' ';
    lo = '0'+(c%10);
  }
  XFillRectangle(dpy,win,contexts[15],
     	         74*6*winscale,winscale*(256+textarea*12)-1,
	         winscale*12+1,12*winscale);
  print_at(74,textarea,hi,0);
  print_at(75,textarea,lo,0);
  UPDATE;
}

void draw_counter(c)
unsigned char c;
{
}

