/*
 * xtool.c
 *
 *      Purpouse        :       Supply graphical aids for applications
 *      Method          :       Ask X11 to draw some polygons in some color
 *      Author          :       Bram Stolk
 *      Date            :       August 1991
 *      Last mod	:       February 1993
 *
 *      This program is a part of the bsxmud client. It communicates with
 *      the X-Window system.
 *
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "config.h"

#include "bitmaps/cursor"
#include "bitmaps/cursormask"

#define SCR_WIDTH	winscale*256*2		/* the size of the window */
#define SCR_HEIGHT	winscale*256

extern void spit_char();
extern void redraw_screen();

char *colornames[16] = { 
				"Black",
				"Blue",
				"ForestGreen",
				"SkyBlue",
				"IndianRed",
				"HotPink",
				"Brown",
				"LightGrey",
				"DimGrey",
				"DeepSkyBlue",
				"Green",
				"Cyan",
				"Tomato",
				"Magenta",
				"Yellow",
				"White"
};

int winscale;			/* Can be either 1 or 2 */
int force;			/* 0 = noforce, 1 =  BW, 2 = Col */
int dont_do_motd;		/* 1 = don't do the extras */
int textarea;			/* nr of lines in textwindow */
Display *dpy;			/* a reference to my display */
Window win;			/* a reference to my window */
int done;			/* boolean: 1 = quit this program */
XGCValues gcvalues;             /* storage for "graphics context" */
int BWMode;			/* are we color or b/w ? */
Font font;			/* font used in textarea + commandline */
unsigned long colors[16];	/* X's approximation to EGA colours */
GC contexts[16];		/* Graphic contexts for each colour/dither */
Pixmap stipplemap[16];		/* For dithering grey scales */
Pixmap buffer;			/* the frame buffer */
Pixmap cursor_pixmap,cursormask_pixmap;
char need_refresh=0;		/* Other process can signal the need */
char brammie[83*141];

/*
 * error() - generic error message handler
 */
int error(message)
char *message;
{
  fprintf(stderr, "bsxclient: %s\n", message);
  exit(-1);
}

/*
 * Whenever changes have been made to either the window, or
 * the background shadow buffer, an update is needed to
 * visualize the changes.
 */
void update_display() {
  /* copy the finished frame buffer to the window */
  XCopyArea(dpy, buffer, win, contexts[0], 0, 0, SCR_WIDTH, winscale*256, 0, 0);
  XFlush(dpy);
}

/*
 * After creating a window, we should wait for the first expose event
 * before drawing anything to the window.
 * stabilize_window does just that.
 */
void stabilize_window(dpy,win)
Display *dpy;
Window win;
{
  XEvent event;
  char gotit=0;

  while (!gotit) {
    XNextEvent(dpy,&event);
    switch(event.type) {
      case Expose:
        if (event.xexpose.window == win) gotit=1;
        break;
      default:
        break;
    }
  }
}

/*
 * This procedure will setup a table of integers that reference the
 * exact colors needed by the application. To do this, it has to allocate
 * the colors in the colormap
 */

void init_colors() {
  int status1,status2;
  Colormap colormap;
  XColor hardwarecolor, rgbcolor;
  int i;

  if (BWMode) {
    /* Monochrome mode */
    char fname[256];
    unsigned dummy_width, dummy_height;
    int dummy_xhot, dummy_yhot;
    for (i=0; i<16; i++) {
      sprintf(fname, "%s/%s%s", STIPPLEDIR, colornames[i], ".xbm");
      if (XReadBitmapFile(dpy, win, fname,
                         &dummy_width, &dummy_height,
                         &(stipplemap[i]),
                         &dummy_xhot, &dummy_yhot)
         != BitmapSuccess)
        error("bsxclient: Error reading bitmap file.");

      gcvalues.foreground = BlackPixel(dpy, DefaultScreen(dpy));
      gcvalues.background = WhitePixel(dpy, DefaultScreen(dpy));
      gcvalues.stipple = stipplemap[i];
      gcvalues.fill_style = FillOpaqueStippled;
      contexts[i]=
        XCreateGC(dpy, win,
		  GCForeground | GCBackground | GCStipple | GCFillStyle,
		  &gcvalues);
    }
    return;
  }

  for (i=0; i<16; i++) {
    colormap= DefaultColormap(dpy, DefaultScreen(dpy));
    status1 = XLookupColor(dpy,colormap,colornames[i],&rgbcolor,&hardwarecolor);
    status2 = XAllocColor(dpy, colormap, &hardwarecolor);
    colors[i] = hardwarecolor.pixel;

    gcvalues.foreground = colors[i];
    gcvalues.background = colors[15];
    contexts[i] = XCreateGC(dpy, win, GCForeground | GCBackground, &gcvalues);
    if ((status1!=1)||(status2!=1))
      error("bsx-client: Could not allocate all 16 colors.");
  }
}

void init_cursor() {
  XColor black,yellow,dummy;
  Cursor cursor;

  cursor_pixmap = 
    XCreateBitmapFromData(dpy,win,cursor_bits,cursor_width,cursor_height);
  cursormask_pixmap = 
    XCreateBitmapFromData(dpy,win,
			  cursormask_bits,cursormask_width,cursor_height);
  XLookupColor(dpy,DefaultColormap(dpy,DefaultScreen(dpy)),
               "black",&dummy,&black);
  XLookupColor(dpy,DefaultColormap(dpy,DefaultScreen(dpy)),
               "yellow",&dummy,&yellow);
  cursor = XCreatePixmapCursor(dpy,cursor_pixmap,cursormask_pixmap,
		      &black,&yellow,
                      cursor_x_hot,cursor_y_hot);
  XDefineCursor(dpy,win,cursor);
}

void put_brammie(xc,yc)
int xc,yc;
{
  int x,y;
  char c;

  for(y=0;y<141;y++)
    for(x=0;x<83;x++)
      if ((c=brammie[x+y*83])!=0) {
        XDrawPoint(dpy,buffer,contexts[c],xc+x,yc+y);
      }
}

char motd[80] = "\" Have a nice virtual session\"";

void intro() {
  FILE *f,*m;
  int size,choice;
  int i;

  f = fopen("bitmaps/head.col", "rb");
  if (f) {
    fread(brammie,83,141,f);
    m = fopen("bitmaps/.motd","rb");
    if (m) {
      /* count the mesgs in .motd */
      size = 0;
      do {
        fgets(motd,80,m);
        size++;
      } while (strcmp(motd,".\n"));
      /* pick one randomly */
      rewind(m);
      srandom(time(NULL));
      choice = random()%size;
      for (i=0; i<=choice; i++)
        fgets(motd,80,m);
    }
    put_brammie(winscale*512-83,0);
    XSetFont(dpy,contexts[0],font);
    XDrawString(dpy,buffer,contexts[0],
	        winscale*20,winscale*110,
	        motd,
	        strlen(motd)-1);
    update_display();
  }
}

Window make_window(title,w,h)
char *title;
int w,h;
{
  int scn;			/* a reference to the screen */
  XSetWindowAttributes attr;	/* storage for window attributes */
  Window win;
  XSizeHints hints;		/* storage for "window hints" */
  int depth;

  scn = DefaultScreen(dpy);
  if (BWMode)
    depth=DefaultDepth(dpy,scn);
  else
    depth=8;
  attr.backing_store = WhenMapped;
  win = XCreateWindow(dpy,
                      DefaultRootWindow(dpy),
                      0,0,
                      w,h,
                      5,depth,CopyFromParent,CopyFromParent,
                      CWBackingStore,&attr);
  XSelectInput(dpy, win, KeyPressMask | ExposureMask);
  XMapWindow(dpy, win);
  stabilize_window(dpy,win);
  hints.flags = PSize | PMinSize | PMaxSize;
  hints.width =hints.min_width =hints.max_width =w;
  hints.height=hints.min_height=hints.max_height=h;
  XSetStandardProperties(dpy,win,title,title,0,NULL,0,&hints);
  attr.backing_store = Always;
  XChangeWindowAttributes(dpy,win,CWBackingStore,&attr);
  return win;
}

/*
 * Do all initialisation necessary to use X
 */
void init_display() {
  int scn;
  char title[80];

  dpy = XOpenDisplay(NULL);	/* connect with the X window system */
  if (dpy == NULL) 
    error("bsx-client: Could not open display.");
  scn = DefaultScreen(dpy);
  if (((DisplayPlanes(dpy,DefaultScreen(dpy))==1) && force==0) || (force==1))
    BWMode=1;
  else
    BWMode=0;
  sprintf(title,"BSX-Client %s",VERSION);
  win = make_window(title,SCR_WIDTH,winscale*(256+(textarea+2)*12));
  buffer = XCreatePixmap(dpy, win, SCR_WIDTH, SCR_HEIGHT, DefaultDepth(dpy, scn));
  if (!buffer) error("bsx-client: Can't create frame buffer.");
  init_colors();
  XFillRectangle(dpy, win, contexts[15], 0, 0, SCR_WIDTH, winscale*(256+(textarea+2)*12));
  XFillRectangle(dpy, buffer, contexts[11],  0, 0, SCR_WIDTH, SCR_HEIGHT);
  update_display();
  if (winscale==1) font = get_font(dpy,6,13);
  if (winscale==2) font = get_font(dpy,12,24);
  redraw_screen();
  if (!BWMode && !dont_do_motd) {
    /*
     * some extras for people with color consoles
     * (free of extra charge)
     */
    init_cursor();
    intro();
  }
}

/*
 * Tell the X server that we're done
 */
void close_display()
{
  int i;

  for (i=0; i<16; i++)
    XFreeGC(dpy,contexts[i]);   /* Free the gc's we`ve used */
  XFreePixmap(dpy, buffer);	/* free the memory we used first */
  if (!BWMode && !dont_do_motd) {
    XFreePixmap(dpy,cursor_pixmap);
    XFreePixmap(dpy,cursormask_pixmap);
  }
  XDestroyWindow(dpy, win);	/* get rid of the window */
  XCloseDisplay(dpy);
}

/*
 * The very core of the entire BSX system:
 * the drawing of 16 colour polygons.
 */
void visualise_poly(p)
struct polygon p;
{
  int i;
  XPoint q[64];

  for (i=0; i<p.size; i++) {
    q[i].x = p.x[i]*winscale*2;
    q[i].y = (255 - p.y[i])*winscale;
  }
  XFillPolygon(dpy, buffer, contexts[p.color], q, p.size, 0, 0);
}

void event_check() {
    XEvent event;
    while (XPending(dpy)) {
	XNextEvent(dpy,&event);
	switch(event.type) {
	case KeyPress:
	    spit_char(event);
	    break;
	case Expose:
	    redraw_screen();
	    update_display();
	    break;
	default:
	    break;
	}
    }
}

void add_line(m)
char *m;
{
}

void do_text(m)
char *m;
{
}

